@@ -37,8 +37,6 @@ from .conftest import asyncio_params, no_other_refs
 if sys.version_info < (3, 11):
     from exceptiongroup import BaseExceptionGroup, ExceptionGroup
 
-pytestmark = pytest.mark.anyio
-
 
 async def async_error(text: str, delay: float = 0.1) -> NoReturn:
     try:
@@ -71,17 +69,18 @@ async def test_success() -> None:
 
 
 @pytest.mark.parametrize(
-    "module",
+    "modulename",
     [
-        pytest.param(asyncio, id="asyncio"),
-        pytest.param(pytest.importorskip("trio"), id="trio"),
+        pytest.param("asyncio"),
+        pytest.param("trio"),
     ],
 )
-def test_run_natively(module: Any) -> None:
+def test_run_natively(modulename: str) -> None:
     async def testfunc() -> None:
         async with create_task_group() as tg:
             tg.start_soon(sleep, 0)
 
+    module = pytest.importorskip(modulename, reason=f"{modulename} is not available")
     if module is asyncio:
         asyncio.run(testfunc())
     else:
@@ -141,7 +140,7 @@ async def test_no_called_started_twice() -> None:
 
 
 async def test_start_with_value() -> None:
-    async def taskfunc(*, task_status: TaskStatus) -> None:
+    async def taskfunc(*, task_status: TaskStatus[str]) -> None:
         task_status.started("foo")
 
     async with create_task_group() as tg:
@@ -161,7 +160,7 @@ async def test_start_crash_before_started_call() -> None:
 
 
 async def test_start_crash_after_started_call() -> None:
-    async def taskfunc(*, task_status: TaskStatus) -> NoReturn:
+    async def taskfunc(*, task_status: TaskStatus[int]) -> NoReturn:
         task_status.started(2)
         raise Exception("foo")
 
@@ -306,7 +305,7 @@ async def test_cancel_with_nested_task_groups() -> None:
 
 
 async def test_start_exception_delivery(anyio_backend_name: str) -> None:
-    def task_fn(*, task_status: TaskStatus = TASK_STATUS_IGNORED) -> None:
+    def task_fn(*, task_status: TaskStatus[str] = TASK_STATUS_IGNORED) -> None:
         task_status.started("hello")
 
     if anyio_backend_name == "trio":
@@ -541,7 +540,13 @@ async def test_cancel_scope_cleared() -> None:
 async def test_fail_after(delay: float) -> None:
     with pytest.raises(TimeoutError):
         with fail_after(delay) as scope:
-            await sleep(1)
+            try:
+                await sleep(1)
+            except get_cancelled_exc_class() as exc:
+                assert "deadline" in str(exc)
+                raise
+            else:
+                pytest.fail("sleep() should have raised a cancellation exception")
 
     assert scope.cancel_called
     assert scope.cancelled_caught
@@ -1059,28 +1064,6 @@ async def test_triple_nested_shield_checkpoint_in_middle() -> None:
     assert not got_past_checkpoint
 
 
-def test_task_group_in_generator(
-    anyio_backend_name: str, anyio_backend_options: dict[str, Any]
-) -> None:
-    async def task_group_generator() -> AsyncGenerator[None, None]:
-        async with create_task_group():
-            yield
-
-    gen = task_group_generator()
-    anyio.run(
-        gen.__anext__,
-        backend=anyio_backend_name,
-        backend_options=anyio_backend_options,
-    )
-    pytest.raises(
-        StopAsyncIteration,
-        anyio.run,
-        gen.__anext__,
-        backend=anyio_backend_name,
-        backend_options=anyio_backend_options,
-    )
-
-
 async def test_exception_group_filtering() -> None:
     """Test that CancelledErrors are filtered out of nested exception groups."""
 
@@ -1822,3 +1805,60 @@ async def test_exception_groups_suppresses_exc_context() -> None:
             raise Exception("Error")
 
     assert exc_info.value.__suppress_context__
+
+
+async def test_cancel_reason() -> None:
+    with CancelScope() as scope:
+        scope.cancel("test reason")
+        with pytest.raises(get_cancelled_exc_class()) as exc_info:
+            await checkpoint()
+
+    task = get_current_task()
+    assert task and task.name
+    exc_info.match("test reason")
+    exc_info.match(task.name)
+
+
+@pytest.mark.parametrize(
+    "native",
+    [
+        pytest.param(False, id="anyio"),
+        pytest.param(True, id="asyncio"),
+    ],
+)
+@pytest.mark.skipif(sys.version_info < (3, 14), reason="Requires Python 3.14")
+@pytest.mark.parametrize("anyio_backend", asyncio_params)
+async def test_asyncio_call_graph(native: bool) -> None:
+    graph: asyncio.FutureCallGraph | None = None
+
+    async def taskfunc(depth: int) -> None:
+        nonlocal graph
+
+        if depth == 2:
+            # The checkpoint is a workaround for eager factories where call graph
+            # capturing doesn't work before the first yield
+            await checkpoint()
+            graph = asyncio.capture_call_graph(asyncio.current_task(), depth=depth)
+            return
+
+        if native:
+            async with asyncio.TaskGroup() as tg:
+                tg.create_task(taskfunc(depth + 1), name=f"depth-{depth + 1}")
+        else:
+            async with create_task_group() as tg:
+                tg.start_soon(taskfunc, depth + 1, name=f"depth-{depth + 1}")
+
+    current_task = asyncio.current_task()
+    assert current_task is not None
+    current_task.set_name("root")
+    await taskfunc(0)
+    assert graph is not None
+    assert isinstance(graph.future, asyncio.Task)
+    task_names: list[str] = [graph.future.get_name()]
+    while graph.awaited_by:
+        assert len(graph.awaited_by) == 1
+        graph = graph.awaited_by[0]
+        assert isinstance(graph.future, asyncio.Task)
+        task_names.append(graph.future.get_name())
+
+    assert task_names == ["depth-2", "depth-1", "root"]
